@@ -776,7 +776,7 @@ def readconf(conf, section_name=None, log_name=None, defaults=None):
     return conf
 
 
-def write_pickle(obj, dest, tmp):
+def write_pickle(obj, dest, tmp=None, pickle_protocol=0):
     """
     Ensure that a pickle file gets written to disk.  The file
     is first written to a tmp location, ensure it is synced to disk, then
@@ -784,11 +784,14 @@ def write_pickle(obj, dest, tmp):
 
     :param obj: python object to be pickled
     :param dest: path of final destination file
-    :param tmp: path to tmp to use
+    :param tmp: path to tmp to use, defaults to None
+    :param pickle_protocol: protocol to pickle the obj with, defaults to 0
     """
-    fd, tmppath = mkstemp(dir=tmp)
+    if tmp == None:
+        tmp = os.path.dirname(dest)
+    fd, tmppath = mkstemp(dir=tmp, suffix='.tmp')
     with os.fdopen(fd, 'wb') as fo:
-        pickle.dump(obj, fo)
+        pickle.dump(obj, fo, pickle_protocol)
         fo.flush()
         os.fsync(fd)
         renamer(tmppath, dest)
