@@ -1,4 +1,4 @@
-# Copyright (c) 2010 OpenStack, LLC.
+# Copyright (c) 2010-2011 OpenStack, LLC.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -24,6 +24,7 @@ import urllib
 
 import simplejson as json
 
+from nose import SkipTest
 from xml.dom import minidom
 
 class AuthenticationFailed(Exception):
@@ -77,13 +78,19 @@ def listing_items(method):
         else:
             items = []
 
+
 class Connection(object):
     def __init__(self, config):
+        for key in 'auth_host auth_port auth_ssl username password'.split():
+            if not config.has_key(key):
+                raise SkipTest
+
         self.auth_host = config['auth_host']
         self.auth_port = int(config['auth_port'])
         self.auth_ssl = config['auth_ssl'] in ('on', 'true', 'yes', '1')
+        self.auth_prefix = config.get('auth_prefix', '/')
 
-        self.account = config['account']
+        self.account = config.get('account')
         self.username = config['username']
         self.password = config['password']
 
@@ -104,12 +111,16 @@ class Connection(object):
             self.storage_token = clone_conn.storage_token
             return
 
+        if self.account:
+            auth_user = '%s:%s' % (self.account, self.username)
+        else:
+            auth_user = self.username
         headers = {
-            'x-storage-user': self.username,
-            'x-storage-pass': self.password,
+            'x-auth-user': auth_user,
+            'x-auth-key': self.password,
         }
 
-        path = '/v1/%s/auth' % (self.account)
+        path = '%sv1.0' % (self.auth_prefix)
         if self.auth_ssl:
             connection = httplib.HTTPSConnection(self.auth_host,
                 port=self.auth_port)
@@ -124,7 +135,7 @@ class Connection(object):
         if response.status == 401:
             raise AuthenticationFailed()
 
-        if response.status != 204:
+        if response.status not in (200, 204):
             raise ResponseError(response)
 
         for hdr in response.getheaders():
